# @Time : 27/02/2023 17:19
# @Author : a.salgas
# @File : market_prices.py
# @Software: PyCharm

from typing import Tuple

import pandas as pd
from aeromaps.models.base import AeromapsModel

from scipy.interpolate import interp1d


class ElectricityCost(AeromapsModel):
    def __init__(self, name="electricity_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        electricity_cost_2020: float = 0.0,
        electricity_cost_2030: float = 0.0,
        electricity_cost_2040: float = 0.0,
        electricity_cost_2050: float = 0.0,
    ) -> Tuple[pd.Series]:
        """LCOE"""

        reference_values_electricity = [
            electricity_cost_2020,
            electricity_cost_2030,
            electricity_cost_2040,
            electricity_cost_2050,
        ]

        reference_years = [2020, 2030, 2040, 2050]

        electricity_price_function = interp1d(
            reference_years, reference_values_electricity, kind="quadratic"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "electricity_market_price"] = electricity_price_function(k)

        electricity_market_price = self.df.loc[:, "electricity_market_price"]

        return electricity_market_price


class CoalCost(AeromapsModel):
    def __init__(self, name="coal_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        coal_cost_2020: float = 0.0,
        coal_cost_2030: float = 0.0,
        coal_cost_2040: float = 0.0,
        coal_cost_2050: float = 0.0,
    ) -> Tuple[pd.Series]:

        reference_values_coal = [coal_cost_2020, coal_cost_2030, coal_cost_2040, coal_cost_2050]

        reference_years = [2020, 2030, 2040, 2050]

        coal_price_function = interp1d(reference_years, reference_values_coal, kind="quadratic")
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "coal_market_price"] = coal_price_function(k)

        coal_market_price = self.df.loc[:, "coal_market_price"]

        return coal_market_price


class GasCost(AeromapsModel):
    def __init__(self, name="gas_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        gas_cost_2020: float = 0.0,
        gas_cost_2030: float = 0.0,
        gas_cost_2040: float = 0.0,
        gas_cost_2050: float = 0.0,
    ) -> Tuple[pd.Series]:

        reference_values_gas = [gas_cost_2020, gas_cost_2030, gas_cost_2040, gas_cost_2050]

        reference_years = [2020, 2030, 2040, 2050]

        gas_price_function = interp1d(reference_years, reference_values_gas, kind="quadratic")
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "gas_market_price"] = gas_price_function(k)

        gas_market_price = self.df.loc[:, "gas_market_price"]

        return gas_market_price


class Co2Cost(AeromapsModel):
    def __init__(self, name="co2_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        co2_cost_2020: float = 0.0,
        co2_cost_2030: float = 0.0,
        co2_cost_2040: float = 0.0,
        co2_cost_2050: float = 0.0,
    ) -> Tuple[pd.Series]:

        reference_values_co2 = [co2_cost_2020, co2_cost_2030, co2_cost_2040, co2_cost_2050]

        reference_years = [2020, 2030, 2040, 2050]

        co2_price_function = interp1d(reference_years, reference_values_co2, kind="linear")
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "co2_market_price"] = co2_price_function(k)

        co2_market_price = self.df.loc[:, "co2_market_price"]

        return co2_market_price


class Co2Tax(AeromapsModel):
    def __init__(self, name="co2_tax", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        co2_tax_2020: float = 0.0,
        co2_tax_2030: float = 0.0,
        co2_tax_2040: float = 0.0,
        co2_tax_2050: float = 0.0,
    ) -> Tuple[pd.Series]:

        reference_values_co2 = [co2_tax_2020, co2_tax_2030, co2_tax_2040, co2_tax_2050]

        reference_years = [2020, 2030, 2040, 2050]

        co2_price_function = interp1d(reference_years, reference_values_co2, kind="linear")
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "carbon_tax"] = co2_price_function(k)

        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "carbon_tax"] = 5.0

        carbon_tax = self.df.loc[:, "carbon_tax"]

        return carbon_tax


class KerosenePrice(AeromapsModel):
    def __init__(self, name="kerosene_market_price", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        kerosene_price_2020: float = 0.0,
        kerosene_price_2030: float = 0.0,
        kerosene_price_2040: float = 0.0,
        kerosene_price_2050: float = 0.0,
    ) -> Tuple[pd.Series]:

        reference_values_kerosene = [
            kerosene_price_2020,
            kerosene_price_2030,
            kerosene_price_2040,
            kerosene_price_2050,
        ]

        reference_years = [2020, 2030, 2040, 2050]

        kerosene_price_function = interp1d(
            reference_years, reference_values_kerosene, kind="quadratic"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "kerosene_market_price"] = kerosene_price_function(k)

        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "kerosene_market_price"] = self.df.loc[2020, "kerosene_market_price"]

        kerosene_market_price = self.df.loc[:, "kerosene_market_price"]

        return kerosene_market_price


class KeroseneCost(AeromapsModel):
    def __init__(self, name="kerosene_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        kerosene_market_price: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_kerosene: pd.Series = pd.Series(dtype="float64"),
        kerosene_emission_factor: pd.Series = pd.Series(dtype="float64"),
        carbon_tax: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series]:
        # kerosene_market_price €/L

        # fuel lower heating value in MJ/L at 15 degrees
        fuel_lhv = 35.3

        kerosene_cost = energy_consumption_kerosene / fuel_lhv * kerosene_market_price / 1000000
        # M€

        # Compute the carbon tax(M€)

        # Kerosene EF is in GCO2e/MJ ; energy consumption in MJ; carbon tax in €/tCO2e ==> conversion in M€

        kerosene_carbon_tax_cost = (
            carbon_tax * kerosene_emission_factor / 1000000 * energy_consumption_kerosene / 1000000
        )

        # Price per litter supplement due to carbon tax [€/L]

        kerosene_price_supplement_carbon_tax = (
            kerosene_carbon_tax_cost / energy_consumption_kerosene * 1000000 * fuel_lhv
        )

        self.df.loc[:, "kerosene_cost"] = kerosene_cost
        self.df.loc[:, "kerosene_carbon_tax_cost"] = kerosene_carbon_tax_cost
        self.df.loc[
            :, "kerosene_price_supplement_carbon_tax"
        ] = kerosene_price_supplement_carbon_tax

        return (kerosene_cost, kerosene_carbon_tax_cost, kerosene_price_supplement_carbon_tax)


class KeroseneBAUCost(AeromapsModel):
    def __init__(self, name="kerosene_BAU_cost", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        kerosene_market_price: pd.Series = pd.Series(dtype="float64"),
        non_discounted_BAU_energy_expenses: pd.Series = pd.Series(dtype="float64"),
        kerosene_emission_factor: pd.Series = pd.Series(dtype="float64"),
        carbon_tax: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series]:
        # kerosene_market_price €/L
        # fuel lower heating value in MJ/L at 15 degrees
        fuel_lhv = 35.3

        # Quantity of kerosene used in BAU (virtual)

        kerosene_consumption_BAU = (
            non_discounted_BAU_energy_expenses / kerosene_market_price * 1000000 * fuel_lhv
        )

        # Carbon tax that would be paid in BAU

        kerosene_carbon_tax_BAU = (
            kerosene_consumption_BAU * carbon_tax * kerosene_emission_factor / 1000000 / 1000000
        )

        self.df.loc[:, "kerosene_carbon_tax_BAU"] = kerosene_carbon_tax_BAU

        return kerosene_carbon_tax_BAU



from typing import Tuple
import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class Temperature(AeromapsModel):
    def __init__(self, name="temperature", *args, **kwargs):
        super().__init__(name, *args, **kwargs)

    def compute(
        self,
        TCRE: float = 0.0,
        temperature_increase_from_aviation_init: pd.Series = pd.Series(dtype="float64"),
        cumulative_total_equivalent_emissions: pd.Series = pd.Series(dtype="float64"),
        cumulative_co2_emissions: pd.Series = pd.Series(dtype="float64"),
        co2_emissions: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series]:
        """Temperature calculation using equivalent emissions."""

        # Total temperature

        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[
                k, "temperature_increase_from_aviation"
            ] = temperature_increase_from_aviation_init.loc[k]

        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "temperature_increase_from_aviation"] = (
                temperature_increase_from_aviation_init.loc[2019]
                + TCRE * cumulative_total_equivalent_emissions.loc[k]
            )

        temperature_increase_from_aviation = self.df["temperature_increase_from_aviation"]

        # Temperature due to CO2 emissions - Assumption of a third of temperature increase due to CO2

        self.df.loc[2019, "temperature_increase_from_co2_from_aviation"] = (
            temperature_increase_from_aviation_init.loc[2019] / 3
        )

        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "temperature_increase_from_co2_from_aviation"] = (
                self.df.loc[2019, "temperature_increase_from_co2_from_aviation"]
                + TCRE * cumulative_co2_emissions.loc[k]
            )

        for k in reversed(range(self.historic_start_year, self.prospection_start_year - 1)):
            self.df.loc[k, "temperature_increase_from_co2_from_aviation"] = (
                self.df.loc[k + 1, "temperature_increase_from_co2_from_aviation"]
                - TCRE * co2_emissions.loc[k] / 1000
            )

        temperature_increase_from_co2_from_aviation = self.df[
            "temperature_increase_from_co2_from_aviation"
        ]

        # Temperature due to non-CO2 effects

        temperature_increase_from_nonco2_from_aviation = (
            temperature_increase_from_aviation - temperature_increase_from_co2_from_aviation
        )

        self.df.loc[
            :, "temperature_increase_from_nonco2_from_aviation"
        ] = temperature_increase_from_nonco2_from_aviation

        return (
            temperature_increase_from_aviation,
            temperature_increase_from_co2_from_aviation,
            temperature_increase_from_nonco2_from_aviation,
        )



from typing import Tuple

import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class DropInFuelConsumption(AeromapsModel):
    def __init__(self, name="drop_in_fuel_consumption", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        biofuel_share: pd.Series = pd.Series(dtype="float64"),
        electrofuel_share: pd.Series = pd.Series(dtype="float64"),
        kerosene_share: pd.Series = pd.Series(dtype="float64"),
        ask_short_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        ask_medium_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        ask_long_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        rtk_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_without_operations_short_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_medium_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_long_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_rtk_without_operations_freight_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_short_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_medium_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_long_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_freight_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
    ]:
        """Drop-in fuel consumption calculation."""

        # WITHOUT OPERATIONS
        # Drop-in fuel
        energy_consumption_short_range_dropin_fuel_without_operations = (
            energy_per_ask_without_operations_short_range_dropin_fuel * ask_short_range_dropin_fuel
        )
        energy_consumption_medium_range_dropin_fuel_without_operations = (
            energy_per_ask_without_operations_medium_range_dropin_fuel
            * ask_medium_range_dropin_fuel
        )
        energy_consumption_long_range_dropin_fuel_without_operations = (
            energy_per_ask_without_operations_long_range_dropin_fuel * ask_long_range_dropin_fuel
        )
        energy_consumption_passenger_dropin_fuel_without_operations = (
            energy_consumption_short_range_dropin_fuel_without_operations
            + energy_consumption_medium_range_dropin_fuel_without_operations
            + energy_consumption_long_range_dropin_fuel_without_operations
        )
        energy_consumption_freight_dropin_fuel_without_operations = (
            energy_per_rtk_without_operations_freight_dropin_fuel * rtk_dropin_fuel
        )
        energy_consumption_dropin_fuel_without_operations = (
            energy_consumption_passenger_dropin_fuel_without_operations
            + energy_consumption_freight_dropin_fuel_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_dropin_fuel_without_operations"
        ] = energy_consumption_short_range_dropin_fuel_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_dropin_fuel_without_operations"
        ] = energy_consumption_medium_range_dropin_fuel_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_dropin_fuel_without_operations"
        ] = energy_consumption_long_range_dropin_fuel_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_dropin_fuel_without_operations"
        ] = energy_consumption_passenger_dropin_fuel_without_operations
        self.df.loc[
            :, "energy_consumption_freight_dropin_fuel_without_operations"
        ] = energy_consumption_freight_dropin_fuel_without_operations
        self.df.loc[
            :, "energy_consumption_dropin_fuel_without_operations"
        ] = energy_consumption_dropin_fuel_without_operations

        # Biofuel
        energy_consumption_short_range_biofuel_without_operations = (
            biofuel_share / 100 * energy_consumption_short_range_dropin_fuel_without_operations
        )
        energy_consumption_medium_range_biofuel_without_operations = (
            biofuel_share / 100 * energy_consumption_medium_range_dropin_fuel_without_operations
        )
        energy_consumption_long_range_biofuel_without_operations = (
            biofuel_share / 100 * energy_consumption_long_range_dropin_fuel_without_operations
        )
        energy_consumption_passenger_biofuel_without_operations = (
            energy_consumption_short_range_biofuel_without_operations
            + energy_consumption_medium_range_biofuel_without_operations
            + energy_consumption_long_range_biofuel_without_operations
        )
        energy_consumption_freight_biofuel_without_operations = (
            biofuel_share / 100 * energy_consumption_freight_dropin_fuel_without_operations
        )
        energy_consumption_biofuel_without_operations = (
            energy_consumption_passenger_biofuel_without_operations
            + energy_consumption_freight_biofuel_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_biofuel_without_operations"
        ] = energy_consumption_short_range_biofuel_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_biofuel_without_operations"
        ] = energy_consumption_medium_range_biofuel_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_biofuel_without_operations"
        ] = energy_consumption_long_range_biofuel_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_biofuel_without_operations"
        ] = energy_consumption_passenger_biofuel_without_operations
        self.df.loc[
            :, "energy_consumption_freight_biofuel_without_operations"
        ] = energy_consumption_freight_biofuel_without_operations
        self.df.loc[
            :, "energy_consumption_biofuel_without_operations"
        ] = energy_consumption_biofuel_without_operations

        # Electrofuel
        energy_consumption_short_range_electrofuel_without_operations = (
            electrofuel_share / 100 * energy_consumption_short_range_dropin_fuel_without_operations
        )
        energy_consumption_medium_range_electrofuel_without_operations = (
            electrofuel_share / 100 * energy_consumption_medium_range_dropin_fuel_without_operations
        )
        energy_consumption_long_range_electrofuel_without_operations = (
            electrofuel_share / 100 * energy_consumption_long_range_dropin_fuel_without_operations
        )
        energy_consumption_passenger_electrofuel_without_operations = (
            energy_consumption_short_range_electrofuel_without_operations
            + energy_consumption_medium_range_electrofuel_without_operations
            + energy_consumption_long_range_electrofuel_without_operations
        )
        energy_consumption_freight_electrofuel_without_operations = (
            electrofuel_share / 100 * energy_consumption_freight_dropin_fuel_without_operations
        )
        energy_consumption_electrofuel_without_operations = (
            energy_consumption_passenger_electrofuel_without_operations
            + energy_consumption_freight_electrofuel_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_electrofuel_without_operations"
        ] = energy_consumption_short_range_electrofuel_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_electrofuel_without_operations"
        ] = energy_consumption_medium_range_electrofuel_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_electrofuel_without_operations"
        ] = energy_consumption_long_range_electrofuel_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_electrofuel_without_operations"
        ] = energy_consumption_passenger_electrofuel_without_operations
        self.df.loc[
            :, "energy_consumption_freight_electrofuel_without_operations"
        ] = energy_consumption_freight_electrofuel_without_operations
        self.df.loc[
            :, "energy_consumption_electrofuel_without_operations"
        ] = energy_consumption_electrofuel_without_operations

        # Kerosene
        energy_consumption_short_range_kerosene_without_operations = (
            kerosene_share / 100 * energy_consumption_short_range_dropin_fuel_without_operations
        )
        energy_consumption_medium_range_kerosene_without_operations = (
            kerosene_share / 100 * energy_consumption_medium_range_dropin_fuel_without_operations
        )
        energy_consumption_long_range_kerosene_without_operations = (
            kerosene_share / 100 * energy_consumption_long_range_dropin_fuel_without_operations
        )
        energy_consumption_passenger_kerosene_without_operations = (
            energy_consumption_short_range_kerosene_without_operations
            + energy_consumption_medium_range_kerosene_without_operations
            + energy_consumption_long_range_kerosene_without_operations
        )
        energy_consumption_freight_kerosene_without_operations = (
            kerosene_share / 100 * energy_consumption_freight_dropin_fuel_without_operations
        )
        energy_consumption_kerosene_without_operations = (
            energy_consumption_passenger_kerosene_without_operations
            + energy_consumption_freight_kerosene_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_kerosene_without_operations"
        ] = energy_consumption_short_range_kerosene_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_kerosene_without_operations"
        ] = energy_consumption_medium_range_kerosene_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_kerosene_without_operations"
        ] = energy_consumption_long_range_kerosene_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_kerosene_without_operations"
        ] = energy_consumption_passenger_kerosene_without_operations
        self.df.loc[
            :, "energy_consumption_freight_kerosene_without_operations"
        ] = energy_consumption_freight_kerosene_without_operations
        self.df.loc[
            :, "energy_consumption_kerosene_without_operations"
        ] = energy_consumption_kerosene_without_operations

        # WITH OPERATIONS
        # Drop-in fuel
        energy_consumption_short_range_dropin_fuel = (
            energy_per_ask_short_range_dropin_fuel * ask_short_range_dropin_fuel
        )
        energy_consumption_medium_range_dropin_fuel = (
            energy_per_ask_medium_range_dropin_fuel * ask_medium_range_dropin_fuel
        )
        energy_consumption_long_range_dropin_fuel = (
            energy_per_ask_long_range_dropin_fuel * ask_long_range_dropin_fuel
        )
        energy_consumption_passenger_dropin_fuel = (
            energy_consumption_short_range_dropin_fuel
            + energy_consumption_medium_range_dropin_fuel
            + energy_consumption_long_range_dropin_fuel
        )
        energy_consumption_freight_dropin_fuel = (
            energy_per_rtk_freight_dropin_fuel * rtk_dropin_fuel
        )
        energy_consumption_dropin_fuel = (
            energy_consumption_passenger_dropin_fuel + energy_consumption_freight_dropin_fuel
        )

        self.df.loc[
            :, "energy_consumption_short_range_dropin_fuel"
        ] = energy_consumption_short_range_dropin_fuel
        self.df.loc[
            :, "energy_consumption_medium_range_dropin_fuel"
        ] = energy_consumption_medium_range_dropin_fuel
        self.df.loc[
            :, "energy_consumption_long_range_dropin_fuel"
        ] = energy_consumption_long_range_dropin_fuel
        self.df.loc[
            :, "energy_consumption_passenger_dropin_fuel"
        ] = energy_consumption_passenger_dropin_fuel
        self.df.loc[
            :, "energy_consumption_freight_dropin_fuel"
        ] = energy_consumption_freight_dropin_fuel
        self.df.loc[:, "energy_consumption_dropin_fuel"] = energy_consumption_dropin_fuel

        # Biofuel
        energy_consumption_short_range_biofuel = (
            biofuel_share / 100 * energy_consumption_short_range_dropin_fuel
        )
        energy_consumption_medium_range_biofuel = (
            biofuel_share / 100 * energy_consumption_medium_range_dropin_fuel
        )
        energy_consumption_long_range_biofuel = (
            biofuel_share / 100 * energy_consumption_long_range_dropin_fuel
        )
        energy_consumption_passenger_biofuel = (
            energy_consumption_short_range_biofuel
            + energy_consumption_medium_range_biofuel
            + energy_consumption_long_range_biofuel
        )
        energy_consumption_freight_biofuel = (
            biofuel_share / 100 * energy_consumption_freight_dropin_fuel
        )
        energy_consumption_biofuel = (
            energy_consumption_passenger_biofuel + energy_consumption_freight_biofuel
        )

        self.df.loc[
            :, "energy_consumption_short_range_biofuel"
        ] = energy_consumption_short_range_biofuel
        self.df.loc[
            :, "energy_consumption_medium_range_biofuel"
        ] = energy_consumption_medium_range_biofuel
        self.df.loc[
            :, "energy_consumption_long_range_biofuel"
        ] = energy_consumption_long_range_biofuel
        self.df.loc[
            :, "energy_consumption_passenger_biofuel"
        ] = energy_consumption_passenger_biofuel
        self.df.loc[:, "energy_consumption_freight_biofuel"] = energy_consumption_freight_biofuel
        self.df.loc[:, "energy_consumption_biofuel"] = energy_consumption_biofuel

        # Electrofuel
        energy_consumption_short_range_electrofuel = (
            electrofuel_share / 100 * energy_consumption_short_range_dropin_fuel
        )
        energy_consumption_medium_range_electrofuel = (
            electrofuel_share / 100 * energy_consumption_medium_range_dropin_fuel
        )
        energy_consumption_long_range_electrofuel = (
            electrofuel_share / 100 * energy_consumption_long_range_dropin_fuel
        )
        energy_consumption_passenger_electrofuel = (
            energy_consumption_short_range_electrofuel
            + energy_consumption_medium_range_electrofuel
            + energy_consumption_long_range_electrofuel
        )
        energy_consumption_freight_electrofuel = (
            electrofuel_share / 100 * energy_consumption_freight_dropin_fuel
        )
        energy_consumption_electrofuel = (
            energy_consumption_passenger_electrofuel + energy_consumption_freight_electrofuel
        )

        self.df.loc[
            :, "energy_consumption_short_range_electrofuel"
        ] = energy_consumption_short_range_electrofuel
        self.df.loc[
            :, "energy_consumption_medium_range_electrofuel"
        ] = energy_consumption_medium_range_electrofuel
        self.df.loc[
            :, "energy_consumption_long_range_electrofuel"
        ] = energy_consumption_long_range_electrofuel
        self.df.loc[
            :, "energy_consumption_passenger_electrofuel"
        ] = energy_consumption_passenger_electrofuel
        self.df.loc[
            :, "energy_consumption_freight_electrofuel"
        ] = energy_consumption_freight_electrofuel
        self.df.loc[:, "energy_consumption_electrofuel"] = energy_consumption_electrofuel

        # Kerosene
        energy_consumption_short_range_kerosene = (
            kerosene_share / 100 * energy_consumption_short_range_dropin_fuel
        )
        energy_consumption_medium_range_kerosene = (
            kerosene_share / 100 * energy_consumption_medium_range_dropin_fuel
        )
        energy_consumption_long_range_kerosene = (
            kerosene_share / 100 * energy_consumption_long_range_dropin_fuel
        )
        energy_consumption_passenger_kerosene = (
            energy_consumption_short_range_kerosene
            + energy_consumption_medium_range_kerosene
            + energy_consumption_long_range_kerosene
        )
        energy_consumption_freight_kerosene = (
            kerosene_share / 100 * energy_consumption_freight_dropin_fuel
        )
        energy_consumption_kerosene = (
            energy_consumption_passenger_kerosene + energy_consumption_freight_kerosene
        )

        self.df.loc[
            :, "energy_consumption_short_range_kerosene"
        ] = energy_consumption_short_range_kerosene
        self.df.loc[
            :, "energy_consumption_medium_range_kerosene"
        ] = energy_consumption_medium_range_kerosene
        self.df.loc[
            :, "energy_consumption_long_range_kerosene"
        ] = energy_consumption_long_range_kerosene
        self.df.loc[
            :, "energy_consumption_passenger_kerosene"
        ] = energy_consumption_passenger_kerosene
        self.df.loc[:, "energy_consumption_freight_kerosene"] = energy_consumption_freight_kerosene
        self.df.loc[:, "energy_consumption_kerosene"] = energy_consumption_kerosene

        return (
            energy_consumption_short_range_dropin_fuel_without_operations,
            energy_consumption_medium_range_dropin_fuel_without_operations,
            energy_consumption_long_range_dropin_fuel_without_operations,
            energy_consumption_passenger_dropin_fuel_without_operations,
            energy_consumption_freight_dropin_fuel_without_operations,
            energy_consumption_dropin_fuel_without_operations,
            energy_consumption_short_range_biofuel_without_operations,
            energy_consumption_medium_range_biofuel_without_operations,
            energy_consumption_long_range_biofuel_without_operations,
            energy_consumption_passenger_biofuel_without_operations,
            energy_consumption_freight_biofuel_without_operations,
            energy_consumption_biofuel_without_operations,
            energy_consumption_short_range_electrofuel_without_operations,
            energy_consumption_medium_range_electrofuel_without_operations,
            energy_consumption_long_range_electrofuel_without_operations,
            energy_consumption_passenger_electrofuel_without_operations,
            energy_consumption_freight_electrofuel_without_operations,
            energy_consumption_electrofuel_without_operations,
            energy_consumption_short_range_kerosene_without_operations,
            energy_consumption_medium_range_kerosene_without_operations,
            energy_consumption_long_range_kerosene_without_operations,
            energy_consumption_passenger_kerosene_without_operations,
            energy_consumption_freight_kerosene_without_operations,
            energy_consumption_kerosene_without_operations,
            energy_consumption_short_range_dropin_fuel,
            energy_consumption_medium_range_dropin_fuel,
            energy_consumption_long_range_dropin_fuel,
            energy_consumption_passenger_dropin_fuel,
            energy_consumption_freight_dropin_fuel,
            energy_consumption_dropin_fuel,
            energy_consumption_short_range_biofuel,
            energy_consumption_medium_range_biofuel,
            energy_consumption_long_range_biofuel,
            energy_consumption_passenger_biofuel,
            energy_consumption_freight_biofuel,
            energy_consumption_biofuel,
            energy_consumption_short_range_electrofuel,
            energy_consumption_medium_range_electrofuel,
            energy_consumption_long_range_electrofuel,
            energy_consumption_passenger_electrofuel,
            energy_consumption_freight_electrofuel,
            energy_consumption_electrofuel,
            energy_consumption_short_range_kerosene,
            energy_consumption_medium_range_kerosene,
            energy_consumption_long_range_kerosene,
            energy_consumption_passenger_kerosene,
            energy_consumption_freight_kerosene,
            energy_consumption_kerosene,
        )


class HydrogenConsumption(AeromapsModel):
    def __init__(self, name="hydrogen_consumption", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        ask_short_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        ask_medium_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        ask_long_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        rtk_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_without_operations_short_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_medium_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_long_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_rtk_without_operations_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_short_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_medium_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_long_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
    ]:
        """Hydrogen consumption calculation."""

        # WITHOUT OPERATIONS
        energy_consumption_short_range_hydrogen_without_operations = (
            energy_per_ask_without_operations_short_range_hydrogen * ask_short_range_hydrogen
        )
        energy_consumption_medium_range_hydrogen_without_operations = (
            energy_per_ask_without_operations_medium_range_hydrogen * ask_medium_range_hydrogen
        )
        energy_consumption_long_range_hydrogen_without_operations = (
            energy_per_ask_without_operations_long_range_hydrogen * ask_long_range_hydrogen
        )
        energy_consumption_passenger_hydrogen_without_operations = (
            energy_consumption_short_range_hydrogen_without_operations
            + energy_consumption_medium_range_hydrogen_without_operations
            + energy_consumption_long_range_hydrogen_without_operations
        )
        energy_consumption_freight_hydrogen_without_operations = (
            energy_per_rtk_without_operations_freight_hydrogen * rtk_hydrogen
        )
        energy_consumption_hydrogen_without_operations = (
            energy_consumption_passenger_hydrogen_without_operations
            + energy_consumption_freight_hydrogen_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_hydrogen_without_operations"
        ] = energy_consumption_short_range_hydrogen_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_hydrogen_without_operations"
        ] = energy_consumption_medium_range_hydrogen_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_hydrogen_without_operations"
        ] = energy_consumption_long_range_hydrogen_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_hydrogen_without_operations"
        ] = energy_consumption_passenger_hydrogen_without_operations
        self.df.loc[
            :, "energy_consumption_freight_hydrogen_without_operations"
        ] = energy_consumption_freight_hydrogen_without_operations
        self.df.loc[
            :, "energy_consumption_hydrogen_without_operations"
        ] = energy_consumption_hydrogen_without_operations

        # WITH OPERATIONS
        energy_consumption_short_range_hydrogen = (
            energy_per_ask_short_range_hydrogen * ask_short_range_hydrogen
        )
        energy_consumption_medium_range_hydrogen = (
            energy_per_ask_medium_range_hydrogen * ask_medium_range_hydrogen
        )
        energy_consumption_long_range_hydrogen = (
            energy_per_ask_long_range_hydrogen * ask_long_range_hydrogen
        )
        energy_consumption_passenger_hydrogen = (
            energy_consumption_short_range_hydrogen
            + energy_consumption_medium_range_hydrogen
            + energy_consumption_long_range_hydrogen
        )
        energy_consumption_freight_hydrogen = energy_per_rtk_freight_hydrogen * rtk_hydrogen
        energy_consumption_hydrogen = (
            energy_consumption_passenger_hydrogen + energy_consumption_freight_hydrogen
        )

        self.df.loc[
            :, "energy_consumption_short_range_hydrogen"
        ] = energy_consumption_short_range_hydrogen
        self.df.loc[
            :, "energy_consumption_medium_range_hydrogen"
        ] = energy_consumption_medium_range_hydrogen
        self.df.loc[
            :, "energy_consumption_long_range_hydrogen"
        ] = energy_consumption_long_range_hydrogen
        self.df.loc[
            :, "energy_consumption_passenger_hydrogen"
        ] = energy_consumption_passenger_hydrogen
        self.df.loc[:, "energy_consumption_freight_hydrogen"] = energy_consumption_freight_hydrogen
        self.df.loc[:, "energy_consumption_hydrogen"] = energy_consumption_hydrogen

        return (
            energy_consumption_short_range_hydrogen_without_operations,
            energy_consumption_medium_range_hydrogen_without_operations,
            energy_consumption_long_range_hydrogen_without_operations,
            energy_consumption_passenger_hydrogen_without_operations,
            energy_consumption_freight_hydrogen_without_operations,
            energy_consumption_hydrogen_without_operations,
            energy_consumption_short_range_hydrogen,
            energy_consumption_medium_range_hydrogen,
            energy_consumption_long_range_hydrogen,
            energy_consumption_passenger_hydrogen,
            energy_consumption_freight_hydrogen,
            energy_consumption_hydrogen,
        )


class EnergyConsumption(AeromapsModel):
    def __init__(self, name="hydrogen_consumption", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        energy_consumption_short_range_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_medium_range_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_long_range_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_short_range_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_medium_range_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_long_range_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_short_range_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_medium_range_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_long_range_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_short_range_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_medium_range_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_long_range_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_biofuel_without_operations: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_electrofuel_without_operations: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_kerosene_without_operations: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_hydrogen_without_operations: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_short_range_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_medium_range_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_long_range_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_short_range_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_medium_range_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_long_range_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_short_range_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_medium_range_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_long_range_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_short_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_medium_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_long_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_hydrogen: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
    ]:
        """Energy consumption calculation."""

        # WITHOUT OPERATIONS
        energy_consumption_short_range_without_operations = (
            energy_consumption_short_range_biofuel_without_operations
            + energy_consumption_short_range_electrofuel_without_operations
            + energy_consumption_short_range_kerosene_without_operations
            + energy_consumption_short_range_hydrogen_without_operations
        )
        energy_consumption_medium_range_without_operations = (
            energy_consumption_medium_range_biofuel_without_operations
            + energy_consumption_medium_range_electrofuel_without_operations
            + energy_consumption_medium_range_kerosene_without_operations
            + energy_consumption_medium_range_hydrogen_without_operations
        )
        energy_consumption_long_range_without_operations = (
            energy_consumption_long_range_biofuel_without_operations
            + energy_consumption_long_range_electrofuel_without_operations
            + energy_consumption_long_range_kerosene_without_operations
            + energy_consumption_long_range_hydrogen_without_operations
        )
        energy_consumption_passenger_without_operations = (
            energy_consumption_passenger_biofuel_without_operations
            + energy_consumption_passenger_electrofuel_without_operations
            + energy_consumption_passenger_kerosene_without_operations
            + energy_consumption_passenger_hydrogen_without_operations
        )
        energy_consumption_freight_without_operations = (
            energy_consumption_freight_biofuel_without_operations
            + energy_consumption_freight_electrofuel_without_operations
            + energy_consumption_freight_kerosene_without_operations
            + energy_consumption_freight_hydrogen_without_operations
        )
        energy_consumption_without_operations = (
            energy_consumption_biofuel_without_operations
            + energy_consumption_electrofuel_without_operations
            + energy_consumption_kerosene_without_operations
            + energy_consumption_hydrogen_without_operations
        )

        self.df.loc[
            :, "energy_consumption_short_range_without_operations"
        ] = energy_consumption_short_range_without_operations
        self.df.loc[
            :, "energy_consumption_medium_range_without_operations"
        ] = energy_consumption_medium_range_without_operations
        self.df.loc[
            :, "energy_consumption_long_range_without_operations"
        ] = energy_consumption_long_range_without_operations
        self.df.loc[
            :, "energy_consumption_passenger_without_operations"
        ] = energy_consumption_passenger_without_operations
        self.df.loc[
            :, "energy_consumption_freight_without_operations"
        ] = energy_consumption_freight_without_operations
        self.df.loc[
            :, "energy_consumption_without_operations"
        ] = energy_consumption_without_operations

        # WITH OPERATIONS
        energy_consumption_short_range = (
            energy_consumption_short_range_biofuel
            + energy_consumption_short_range_electrofuel
            + energy_consumption_short_range_kerosene
            + energy_consumption_short_range_hydrogen
        )
        energy_consumption_medium_range = (
            energy_consumption_medium_range_biofuel
            + energy_consumption_medium_range_electrofuel
            + energy_consumption_medium_range_kerosene
            + energy_consumption_medium_range_hydrogen
        )
        energy_consumption_long_range = (
            energy_consumption_long_range_biofuel
            + energy_consumption_long_range_electrofuel
            + energy_consumption_long_range_kerosene
            + energy_consumption_long_range_hydrogen
        )
        energy_consumption_passenger = (
            energy_consumption_passenger_biofuel
            + energy_consumption_passenger_electrofuel
            + energy_consumption_passenger_kerosene
            + energy_consumption_passenger_hydrogen
        )
        energy_consumption_freight = (
            energy_consumption_freight_biofuel
            + energy_consumption_freight_electrofuel
            + energy_consumption_freight_kerosene
            + energy_consumption_freight_hydrogen
        )
        energy_consumption = (
            energy_consumption_biofuel
            + energy_consumption_electrofuel
            + energy_consumption_kerosene
            + energy_consumption_hydrogen
        )

        self.df.loc[:, "energy_consumption_short_range"] = energy_consumption_short_range
        self.df.loc[:, "energy_consumption_medium_range"] = energy_consumption_medium_range
        self.df.loc[:, "energy_consumption_long_range"] = energy_consumption_long_range
        self.df.loc[:, "energy_consumption_passenger"] = energy_consumption_passenger
        self.df.loc[:, "energy_consumption_freight"] = energy_consumption_freight
        self.df.loc[:, "energy_consumption"] = energy_consumption

        return (
            energy_consumption_short_range_without_operations,
            energy_consumption_medium_range_without_operations,
            energy_consumption_long_range_without_operations,
            energy_consumption_passenger_without_operations,
            energy_consumption_freight_without_operations,
            energy_consumption_without_operations,
            energy_consumption_short_range,
            energy_consumption_medium_range,
            energy_consumption_long_range,
            energy_consumption_passenger,
            energy_consumption_freight,
            energy_consumption,
        )



import matplotlib.pyplot as plt
import numpy as np
from .constants import plot_3_x, plot_3_y


class CumulativeCO2EmissionsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_cumulative_co2_emissions,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "cumulative_co2_emissions"],
            color="blue",
            linestyle="-",
            label="Cumulative CO2 emissions",
            linewidth=2,
        )

        (self.line_aviation_carbon_budget,) = self.ax.plot(
            self.prospective_years,
            np.ones(len(self.prospective_years)) * self.float_outputs["aviation_carbon_budget"],
            color="black",
            linestyle="--",
            label="Aviation carbon budget",
            linewidth=1,
        )

        self.ax.grid()
        self.ax.set_title(
            "Comparison of cumulative CO2 emissions from air transport\nwith allocated carbon budget (from 2019)"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Cumulative CO2 emissions [GtCO2]")
        self.ax.legend()
        self.ax = plt.gca()
        self.ax.set_xlim(2019, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_cumulative_co2_emissions.set_ydata(
            self.df.loc[self.prospective_years, "cumulative_co2_emissions"]
        )

        self.line_aviation_carbon_budget.set_ydata(
            np.ones(len(self.prospective_years)) * self.float_outputs["aviation_carbon_budget"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DirectH2OEmissionsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "h2o_emissions"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_h2o_emissions,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2o_emissions"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of direct H2O emissions\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Direct H2O emissions [MtH2O]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_h2o_emissions.set_ydata(self.df.loc[self.prospective_years, "h2o_emissions"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DirectNOxEmissionsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "nox_emissions"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_nox_emissions,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "nox_emissions"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of direct NOx emissions\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Direct NOx emissions [MtNOx]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_nox_emissions.set_ydata(self.df.loc[self.prospective_years, "nox_emissions"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DirectSulfurEmissionsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "sulfur_emissions"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_sulfur_emissions,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "sulfur_emissions"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of direct sulfur emissions\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Direct sulfur emissions [MtSO2]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_sulfur_emissions.set_ydata(
            self.df.loc[self.prospective_years, "sulfur_emissions"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DirectSootEmissionsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "soot_emissions"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_soot_emissions,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "soot_emissions"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of direct soot emissions\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Direct soot emissions [MtBC]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_soot_emissions.set_ydata(self.df.loc[self.prospective_years, "soot_emissions"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()



from typing import Tuple

import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class CarbonBudgetConsumedShare(AeromapsModel):
    def __init__(self, name="carbon_budget_consumed_share", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        cumulative_co2_emissions: pd.Series = pd.Series(dtype="float64"),
        cumulative_total_equivalent_emissions: pd.Series = pd.Series(dtype="float64"),
        gross_carbon_budget_2050: float = 0.0,
        equivalent_gross_carbon_budget_2050: float = 0.0,
    ) -> Tuple[float, float]:
        """(Equivalent) Carbon budget consumption share calculation."""

        carbon_budget_consumed_share = (
            cumulative_co2_emissions.loc[self.end_year] / gross_carbon_budget_2050 * 100
        )
        equivalent_carbon_budget_consumed_share = (
            cumulative_total_equivalent_emissions.loc[self.end_year]
            / equivalent_gross_carbon_budget_2050
            * 100
        )

        self.float_outputs["carbon_budget_consumed_share"] = carbon_budget_consumed_share
        self.float_outputs[
            "equivalent_carbon_budget_consumed_share"
        ] = equivalent_carbon_budget_consumed_share

        return carbon_budget_consumed_share, equivalent_carbon_budget_consumed_share


class ResourcesConsumedShare(AeromapsModel):
    def __init__(self, name="resources_consumed_share", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        biomass_consumption_end_year: float = 0.0,
        electricity_consumption_end_year: float = 0.0,
        available_biomass_total: float = 0.0,
        available_electricity: float = 0.0,
    ) -> Tuple[float, float]:
        """Resources consumption share calculation."""

        biomass_consumed_share = biomass_consumption_end_year / available_biomass_total * 100
        electricity_consumed_share = electricity_consumption_end_year / available_electricity * 100

        self.float_outputs["biomass_consumed_share"] = biomass_consumed_share
        self.float_outputs["electricity_consumed_share"] = electricity_consumed_share

        return biomass_consumed_share, electricity_consumed_share



from aenum import Enum


# Definition of energy types
class EnergyTypes(Enum):
    """
    Enumeration of Energy Types.
    """

    DROP_IN_FUEL = "Drop-in Fuel"
    HYDROGENE = "Hydrogene"


# Definition of fuel types
class FuelTypes(Enum):
    """
    Enumeration of Fuel Types.
    """

    KEROSENE = "Kerosene"
    BIOFUEL = "Biofuel"
    ELECTROFUEL = "Electrofuel"



from typing import Tuple

import numpy as np
import pandas as pd
from scipy.interpolate import interp1d

from aeromaps.models.base import AeromapsModel


class BiofuelProduction(AeromapsModel):
    def __init__(self, name="biofuel_production", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        biofuel_hefa_fog_share_2020: float,
        biofuel_hefa_fog_share_2030: float,
        biofuel_hefa_fog_share_2040: float,
        biofuel_hefa_fog_share_2050: float,
        biofuel_hefa_others_share_2020: float,
        biofuel_hefa_others_share_2030: float,
        biofuel_hefa_others_share_2040: float,
        biofuel_hefa_others_share_2050: float,
        biofuel_ft_others_share_2020: float,
        biofuel_ft_others_share_2030: float,
        biofuel_ft_others_share_2040: float,
        biofuel_ft_others_share_2050: float,
        biofuel_ft_msw_share_2020: float,
        biofuel_ft_msw_share_2030: float,
        biofuel_ft_msw_share_2040: float,
        biofuel_ft_msw_share_2050: float,
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series, pd.Series]:
        """Biomass production calculation using interpolation functions"""

        reference_years = [2020, 2030, 2040, self.parameters.end_year]

        # HEFA FOG
        reference_values_hefa_fog = [
            biofuel_hefa_fog_share_2020,
            biofuel_hefa_fog_share_2030,
            biofuel_hefa_fog_share_2040,
            biofuel_hefa_fog_share_2050,
        ]
        biofuel_hefa_fog_share_function = interp1d(
            reference_years, reference_values_hefa_fog, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "biofuel_hefa_fog_share"] = biofuel_hefa_fog_share_function(k)

        # HEFA OTHERS
        reference_values_hefa_others = [
            biofuel_hefa_others_share_2020,
            biofuel_hefa_others_share_2030,
            biofuel_hefa_others_share_2040,
            biofuel_hefa_others_share_2050,
        ]
        biofuel_hefa_others_share_function = interp1d(
            reference_years, reference_values_hefa_others, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "biofuel_hefa_others_share"] = biofuel_hefa_others_share_function(k)

        # FT OTHERS
        reference_values_ft_others = [
            biofuel_ft_others_share_2020,
            biofuel_ft_others_share_2030,
            biofuel_ft_others_share_2040,
            biofuel_ft_others_share_2050,
        ]
        biofuel_ft_others_share_function = interp1d(
            reference_years, reference_values_ft_others, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "biofuel_ft_others_share"] = biofuel_ft_others_share_function(k)

        # FT MSW
        reference_values_ft_msw = [
            biofuel_ft_msw_share_2020,
            biofuel_ft_msw_share_2030,
            biofuel_ft_msw_share_2040,
            biofuel_ft_msw_share_2050,
        ]
        biofuel_ft_msw_share_function = interp1d(
            reference_years, reference_values_ft_msw, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "biofuel_ft_msw_share"] = biofuel_ft_msw_share_function(k)

        biofuel_hefa_fog_share = self.df["biofuel_hefa_fog_share"]
        biofuel_hefa_others_share = self.df["biofuel_hefa_others_share"]
        biofuel_ft_others_share = self.df["biofuel_ft_others_share"]
        biofuel_ft_msw_share = self.df["biofuel_ft_msw_share"]

        # ATJ
        biofuel_atj_share = (
            100
            - biofuel_hefa_fog_share
            - biofuel_hefa_others_share
            - biofuel_ft_others_share
            - biofuel_ft_msw_share
        )
        self.df.loc[:, "biofuel_atj_share"] = biofuel_atj_share

        return (
            biofuel_hefa_fog_share,
            biofuel_hefa_others_share,
            biofuel_ft_others_share,
            biofuel_ft_msw_share,
            biofuel_atj_share,
        )


class HydrogenProduction(AeromapsModel):
    def __init__(self, name="hydrogen_production", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        hydrogen_electrolysis_share_2020: float = 0.0,
        hydrogen_electrolysis_share_2030: float = 0.0,
        hydrogen_electrolysis_share_2040: float = 0.0,
        hydrogen_electrolysis_share_2050: float = 0.0,
        hydrogen_gas_ccs_share_2020: float = 0.0,
        hydrogen_gas_ccs_share_2030: float = 0.0,
        hydrogen_gas_ccs_share_2040: float = 0.0,
        hydrogen_gas_ccs_share_2050: float = 0.0,
        hydrogen_coal_ccs_share_2020: float = 0.0,
        hydrogen_coal_ccs_share_2030: float = 0.0,
        hydrogen_coal_ccs_share_2040: float = 0.0,
        hydrogen_coal_ccs_share_2050: float = 0.0,
        hydrogen_gas_share_2020: float = 0.0,
        hydrogen_gas_share_2030: float = 0.0,
        hydrogen_gas_share_2040: float = 0.0,
        hydrogen_gas_share_2050: float = 0.0,
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series, pd.Series]:
        """Hydrogen production calculation using interpolation functions"""

        reference_years = [2020, 2030, 2040, self.parameters.end_year]

        # Electrolysis
        reference_values_electrolysis = [
            hydrogen_electrolysis_share_2020,
            hydrogen_electrolysis_share_2030,
            hydrogen_electrolysis_share_2040,
            hydrogen_electrolysis_share_2050,
        ]
        hydrogen_electrolysis_share_function = interp1d(
            reference_years, reference_values_electrolysis, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "hydrogen_electrolysis_share"] = hydrogen_electrolysis_share_function(k)

        # Gas CCS
        reference_values_gas_ccs = [
            hydrogen_gas_ccs_share_2020,
            hydrogen_gas_ccs_share_2030,
            hydrogen_gas_ccs_share_2040,
            hydrogen_gas_ccs_share_2050,
        ]
        hydrogen_gas_ccs_share_function = interp1d(
            reference_years, reference_values_gas_ccs, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "hydrogen_gas_ccs_share"] = hydrogen_gas_ccs_share_function(k)

        # Coal CCS
        reference_values_coal_ccs = [
            hydrogen_coal_ccs_share_2020,
            hydrogen_coal_ccs_share_2030,
            hydrogen_coal_ccs_share_2040,
            hydrogen_coal_ccs_share_2050,
        ]
        hydrogen_coal_ccs_share_function = interp1d(
            reference_years, reference_values_coal_ccs, kind="linear"
        )
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "hydrogen_coal_ccs_share"] = hydrogen_coal_ccs_share_function(k)

        # Gas
        reference_values_gas = [
            hydrogen_gas_share_2020,
            hydrogen_gas_share_2030,
            hydrogen_gas_share_2040,
            hydrogen_gas_share_2050,
        ]
        hydrogen_gas_share_function = interp1d(reference_years, reference_values_gas, kind="linear")
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "hydrogen_gas_share"] = hydrogen_gas_share_function(k)

        hydrogen_electrolysis_share = self.df["hydrogen_electrolysis_share"]
        hydrogen_gas_ccs_share = self.df["hydrogen_gas_ccs_share"]
        hydrogen_coal_ccs_share = self.df["hydrogen_coal_ccs_share"]
        hydrogen_gas_share = self.df["hydrogen_gas_share"]

        # Coal
        hydrogen_coal_share = (
            100
            - hydrogen_electrolysis_share
            - hydrogen_gas_ccs_share
            - hydrogen_coal_ccs_share
            - hydrogen_gas_share
        )
        self.df.loc[:, "hydrogen_coal_share"] = hydrogen_coal_share

        return (
            hydrogen_electrolysis_share,
            hydrogen_gas_ccs_share,
            hydrogen_coal_ccs_share,
            hydrogen_gas_share,
            hydrogen_coal_share,
        )



from aeromaps.models.impacts.costs.operations.average_ops_cost import (
    PassengerAircraftDocEnergy,
    DropInMeanMfsp,
    PassengerAircraftDocCarbonTax,
    PassengerAircraftDocNonEnergyComplex,
    PassengerAircraftTotalDoc,
    PassengerAircraftDocNonEnergySimple,
)
from aeromaps.models.impacts.energy_resources.abatement_potential import BiofuelAbatementPotential

from aeromaps.models.air_transport.air_traffic.rpk import (
    RPK,
    RPKReference,
    RPKMeasures,
)
from aeromaps.models.air_transport.air_traffic.rtk import RTK, RTKReference
from aeromaps.models.air_transport.air_traffic.total_aircraft_distance import TotalAircraftDistance
from aeromaps.models.air_transport.aircraft_fleet_and_operations.load_factor.load_factor import (
    LoadFactor,
)
from aeromaps.models.air_transport.air_traffic.ask import ASK
from aeromaps.models.air_transport.aircraft_fleet_and_operations.operations.operations import (
    OperationsSimple,
)
from aeromaps.models.air_transport.aircraft_fleet_and_operations.non_co2.non_co2 import (
    OperationsContrailsSimple,
)
from aeromaps.models.air_transport.aircraft_fleet_and_operations.fleet.aircraft_efficiency import (
    PassengerAircraftEfficiencySimple,
    PassengerAircraftEfficiencyComplex,
    FreightAircraftEfficiency,
)
from aeromaps.models.air_transport.aircraft_fleet_and_operations.aircraft_fleet_and_operations import (
    EnergyIntensity,
)
from aeromaps.models.air_transport.aircraft_energy.fuel_distribution import FuelDistribution
from aeromaps.models.sustainability_assessment.climate.carbon_budgets import GrossCarbonBudget
from aeromaps.models.sustainability_assessment.climate.equivalent_carbon_budgets import (
    EquivalentGrossCarbonBudget,
)
from aeromaps.models.air_transport.aircraft_energy.efficiency import (
    BiofuelEfficiency,
    ElectricityBasedFuelEfficiency,
)
from aeromaps.models.air_transport.aircraft_energy.fuel_emissions import (
    BiofuelEmissionFactor,
    ElectricityEmissionFactor,
    HydrogenEmissionFactor,
    ElectrofuelEmissionFactor,
    KeroseneEmissionFactor,
)
from aeromaps.models.air_transport.aircraft_energy.production_choices import (
    BiofuelProduction,
    HydrogenProduction,
)
from aeromaps.models.sustainability_assessment.energy.resources_availability import (
    BiomassAvailability,
    ElectricityAvailability,
)

from aeromaps.models.impacts.effective_radiative_forcing.effective_radiative_forcing import (
    ERF,
    DetailedERF,
)
from aeromaps.models.impacts.emissions.co2_emissions import (
    CO2Emissions,
    KayaFactors,
    CumulativeCO2Emissions,
    DetailedCo2Emissions,
    DetailedCumulativeCO2Emissions,
)
from aeromaps.models.impacts.emissions.non_co2_emissions import NOxEmissionIndex, NonCO2Emissions
from aeromaps.models.impacts.energy_resources.energy_consumption import (
    DropInFuelConsumption,
    HydrogenConsumption,
    EnergyConsumption,
)
from aeromaps.models.impacts.energy_resources.resources_consumption import (
    BiomassConsumption,
    ElectricityConsumption,
)
from aeromaps.models.impacts.equivalent_co2_emissions.equivalent_co2_emissions import (
    EquivalentCO2Emissions,
)
from aeromaps.models.impacts.climate.climate import Temperature
from aeromaps.models.impacts.others.others import (
    EmissionsPerRPK,
    EmissionsPerRTK,
    DropinFuelConsumptionLiterPerPax100km,
)
from aeromaps.models.impacts.others.comparison import (
    CarbonBudgetConsumedShare,
    ResourcesConsumedShare,
)


# COSTS
from aeromaps.models.impacts.costs.energy.biofuel import BiofuelCost, BiofuelMfsp, BiofuelCapex
from aeromaps.models.impacts.costs.energy.market_prices import (
    ElectricityCost,
    Co2Cost,
    KeroseneCost,
    KerosenePrice,
    Co2Tax,
    KeroseneBAUCost,
    CoalCost,
    GasCost,
)
from aeromaps.models.impacts.costs.energy.power_to_liquid import (
    ElectrofuelCost,
    ElectrofuelCapex,
    ElectrofuelFixedOpex,
    ElectrofuelVarOpex,
    # ElectrofuelSpecificElectricity,
    ElectrofuelSpecificCo2,
)

from aeromaps.models.impacts.costs.energy.liquid_hydrogen import (
    LiquidHydrogenCost,
    ElectrolyserCapex,
    ElectrolyserFixedOpex,
    ElectrolyserVarOpex,
    # ElectrolyserSpecificElectricity,
    LiquefierCapex,
    CcsCost,
    CoalEfficiency,
    CoalFixedOpex,
    CoalCapex,
    CoalCcsEfficiency,
    CoalCcsFixedOpex,
    CoalCcsCapex,
    GasEfficiency,
    GasFixedOpex,
    GasCapex,
    GasCcsEfficiency,
    GasCcsFixedOpex,
    GasCcsCapex,
    # LiquefierSpecificElectricity,
)

from aeromaps.models.impacts.costs.scenario.scenario_cost import (
    DicountedScenarioCost,
    NonDiscountedScenarioCost,
)


models_simple = {
    "rpk_measures": RPKMeasures("rpk_measures"),
    "rpk": RPK("rpk"),
    "rpk_reference": RPKReference("rpk_reference"),
    "total_aircraft_distance": TotalAircraftDistance("total_aircraft_distance"),
    "rtk": RTK("rtk"),
    "rtk_reference": RTKReference("rtk_reference"),
    # "short_range_distribution": ShortRangeDistribution(
    #     "short_range_distribution"
    # ),
    # "rpk_short_range": RPKShortRange(
    #     "rpk_short_range"
    # ),
    "load_factor": LoadFactor("load_factor"),
    "ask": ASK("ask"),
    "operations_simple": OperationsSimple("operations_simple"),
    "operations_contrails_simple": OperationsContrailsSimple("operations_contrails_simple"),
    "passenger_aircraft_efficiency_simple": PassengerAircraftEfficiencySimple(
        "passenger_aircraft_efficiency_simple"
    ),
    "freight_aircraft_efficiency": FreightAircraftEfficiency("freight_aircraft_efficiency"),
    "energy_intensity": EnergyIntensity("energy_intensity"),
    "aircraft_energy": FuelDistribution("aircraft_energy"),
    "gross_carbon_budget": GrossCarbonBudget("gross_carbon_budget"),
    "equivalent_gross_carbon_budget": EquivalentGrossCarbonBudget("equivalent_gross_carbon_budget"),
    "biofuel_efficiency": BiofuelEfficiency("biofuel_efficiency"),
    "electricity_based_fuel_efficiency": ElectricityBasedFuelEfficiency(
        "electricity_based_fuel_efficiency"
    ),
    "biofuel_emission_factor": BiofuelEmissionFactor("biofuel_emission_factor"),
    "electricity_emission_factor": ElectricityEmissionFactor("electricity_emission_factor"),
    "hydrogen_emission_factor": HydrogenEmissionFactor("hydrogen_emission_factor"),
    "electrofuel_emission_factor": ElectrofuelEmissionFactor("electrofuel_emission_factor"),
    "kerosene_emission_factor": KeroseneEmissionFactor("kerosene_emission_factor"),
    "biofuel_production": BiofuelProduction("biofuel_production"),
    "hydrogen_production": HydrogenProduction("hydrogen_production"),
    "biomass_availability": BiomassAvailability("biomass_availability"),
    "electricity_availability": ElectricityAvailability("electricity_availability"),
    "effective_radiative_forcing": ERF("effective_radiative_forcing"),
    "detailed_erf": DetailedERF("detailed_erf"),
    "kaya_factors": KayaFactors("kaya_factors"),
    "co2_emissions": CO2Emissions("co2_emissions"),
    "cumulative_co2_emissions": CumulativeCO2Emissions("cumulative_co2_emissions"),
    "detailed_co2_emissions": DetailedCo2Emissions("detailed_co2_emissions"),
    "detailed_cumulative_co2_emissions": DetailedCumulativeCO2Emissions(
        "detailed_cumulative_co2_emissions"
    ),
    "nox_emission_index": NOxEmissionIndex("nox_emission_index"),
    "non_co2_emissions": NonCO2Emissions("non_co2_emissions"),
    "drop_in_fuel_consumption": DropInFuelConsumption("drop_in_fuel_consumption"),
    "hydrogen_consumption": HydrogenConsumption("hydrogen_consumption"),
    "energy_consumption": EnergyConsumption("energy_consumption"),
    "biomass_consumption": BiomassConsumption("biomass_consumption"),
    "electricity_consumption": ElectricityConsumption("electricity_consumption"),
    "equivalent_co2_emissions": EquivalentCO2Emissions("equivalent_co2_emissions"),
    "temperature": Temperature("temperature"),
    "emissions_per_rpk": EmissionsPerRPK("emissions_per_rpk"),
    "emissions_per_rtk": EmissionsPerRTK("emissions_per_rtk"),
    "dropin_fuel_consumption_liter_per_pax_100km": DropinFuelConsumptionLiterPerPax100km(
        "dropin_fuel_consumption_liter_per_pax_100km"
    ),
    "carbon_budget_consumed_share": CarbonBudgetConsumedShare("carbon_budget_consumed_share"),
    "resources_consumed_share": ResourcesConsumedShare("resources_consumed_share"),
    "biofuel_mfsp": BiofuelMfsp("biofuel_mfsp"),
    "biofuel_capex": BiofuelCapex("biofuel_capex"),
    "kerosene_market_price": KerosenePrice("kerosene_market_price"),
    "kerosene_cost": KeroseneCost("kerosene_cost"),
    "kerosene_BAU_cost": KeroseneBAUCost("kerosene_BAU_cost"),
    "biofuel_cost": BiofuelCost("biofuel_cost"),
    "co2_cost": Co2Cost("co2_cost"),
    "co2_tax": Co2Tax("co2_tax"),
    "electricity_cost": ElectricityCost("electricity_cost"),
    "coal_cost": CoalCost("coal_cost"),
    "gas_cost": GasCost("coal_cost"),
    "liquid_hydrogen_cost": LiquidHydrogenCost("liquid_hydrogen_cost"),
    "electrolyser_capex": ElectrolyserCapex("electrolyser_capex"),
    "electrolyser_fixed_opex": ElectrolyserFixedOpex("electrolyser_fixed_opex"),
    "electrolyser_var_opex": ElectrolyserVarOpex("electrolyser_var_opex"),
    # "electrolyser_specific_electricity": ElectrolyserSpecificElectricity(
    #     "electrolyser_specific_electricity"
    # ),
    "gas_ccs_capex": GasCcsCapex("gas_ccs_capex"),
    "gas_ccs_fixed_opex": GasCcsFixedOpex("gas_ccs_fixed_opex"),
    "gas_ccs_efficiency": GasCcsEfficiency("gas_ccs_efficiency"),
    "gas_capex": GasCapex("gas_capex"),
    "gas_fixed_opex": GasFixedOpex("gas_fixed_opex"),
    "gas_efficiency": GasEfficiency("gas_efficiency"),
    "coal_ccs_capex": CoalCcsCapex("coal_ccs_capex"),
    "coal_ccs_fixed_opex": CoalCcsFixedOpex("coal_ccs_fixed_opex"),
    "coal_ccs_efficiency": CoalCcsEfficiency("coal_ccs_efficiency"),
    "coal_capex": CoalCapex("coal_capex"),
    "coal_fixed_opex": CoalFixedOpex("coal_fixed_opex"),
    "coal_efficiency": CoalEfficiency("coal_efficiency"),
    "ccs_cost": CcsCost("ccs_cost"),
    "liquefier_capex": LiquefierCapex("liquefier_capex"),
    # "liquefier_specific_electricity": LiquefierSpecificElectricity(
    #     "liquefier_specific_electricity"
    # ),
    "electrofuel_cost": ElectrofuelCost("electrofuel_cost"),
    "electrofuel_capex": ElectrofuelCapex("electrofuel_capex"),
    "electrofuel_fixed_opex": ElectrofuelFixedOpex("electrofuel_fixed_opex"),
    "electrofuel_var_opex": ElectrofuelVarOpex("electrofuel_var_opex"),
    # "electrofuel_specific_electricity": ElectrofuelSpecificElectricity("electrofuel_specific_electricity"),
    "electrofuel_specific_co2": ElectrofuelSpecificCo2("electrofuel_specific_co2"),
    "discounted_scenario_cost": DicountedScenarioCost("discounted_scenario_cost"),
    "non_discounted_scenario_cost": NonDiscountedScenarioCost("non_discounted_scenario_cost"),
    "biofuel_abatement_potential": BiofuelAbatementPotential("biofuel_abatement_potential"),
    "passenger_aircraft_doc_energy": PassengerAircraftDocEnergy("passenger_aircraft_doc_energy"),
    "passenger_aircraft_doc_non_energy_simple": PassengerAircraftDocNonEnergySimple(
        "passenger_aircraft_doc_non_energy_simple"
    ),
    "passenger_aircraft_total_doc": PassengerAircraftTotalDoc("passenger_aircraft_total_doc"),
    "dropin_mean_mfsp": DropInMeanMfsp("dropin_mean_mfsp"),
    "passenger_aircraft_doc_carbon_tax": PassengerAircraftDocCarbonTax(
        "passenger_aircraft_doc_carbon_tax"
    ),
}


models_complex = {
    "rpk_measures": RPKMeasures("rpk_measures"),
    "rpk": RPK("rpk"),
    "rpk_reference": RPKReference("rpk_reference"),
    "total_aircraft_distance": TotalAircraftDistance("total_aircraft_distance"),
    "rtk": RTK("rtk"),
    "rtk_reference": RTKReference("rtk_reference"),
    # "short_range_distribution": ShortRangeDistribution(
    #     "short_range_distribution"
    # ),
    # "rpk_short_range": RPKShortRange(
    #     "rpk_short_range"
    # ),
    "load_factor": LoadFactor("load_factor"),
    "ask": ASK("ask"),
    "operations_simple": OperationsSimple("operations_simple"),
    "operations_contrails_simple": OperationsContrailsSimple("operations_contrails_simple"),
    "passenger_aircraft_efficiency_complex": PassengerAircraftEfficiencyComplex(
        "passenger_aircraft_efficiency_complex"
    ),
    "freight_aircraft_efficiency": FreightAircraftEfficiency("freight_aircraft_efficiency"),
    "energy_intensity": EnergyIntensity("energy_intensity"),
    "aircraft_energy": FuelDistribution("aircraft_energy"),
    "gross_carbon_budget": GrossCarbonBudget("gross_carbon_budget"),
    "equivalent_gross_carbon_budget": EquivalentGrossCarbonBudget("equivalent_gross_carbon_budget"),
    "biofuel_efficiency": BiofuelEfficiency("biofuel_efficiency"),
    "electricity_based_fuel_efficiency": ElectricityBasedFuelEfficiency(
        "electricity_based_fuel_efficiency"
    ),
    "biofuel_emission_factor": BiofuelEmissionFactor("biofuel_emission_factor"),
    "electricity_emission_factor": ElectricityEmissionFactor("electricity_emission_factor"),
    "hydrogen_emission_factor": HydrogenEmissionFactor("hydrogen_emission_factor"),
    "electrofuel_emission_factor": ElectrofuelEmissionFactor("electrofuel_emission_factor"),
    "kerosene_emission_factor": KeroseneEmissionFactor("kerosene_emission_factor"),
    "biofuel_production": BiofuelProduction("biofuel_production"),
    "hydrogen_production": HydrogenProduction("hydrogen_production"),
    "biomass_availability": BiomassAvailability("biomass_availability"),
    "electricity_availability": ElectricityAvailability("electricity_availability"),
    "effective_radiative_forcing": ERF("effective_radiative_forcing"),
    "detailed_erf": DetailedERF("detailed_erf"),
    "kaya_factors": KayaFactors("kaya_factors"),
    "co2_emissions": CO2Emissions("co2_emissions"),
    "cumulative_co2_emissions": CumulativeCO2Emissions("cumulative_co2_emissions"),
    "detailed_co2_emissions": DetailedCo2Emissions("detailed_co2_emissions"),
    "detailed_cumulative_co2_emissions": DetailedCumulativeCO2Emissions(
        "detailed_cumulative_co2_emissions"
    ),
    "nox_emission_index": NOxEmissionIndex("nox_emission_index"),
    "non_co2_emissions": NonCO2Emissions("non_co2_emissions"),
    "drop_in_fuel_consumption": DropInFuelConsumption("drop_in_fuel_consumption"),
    "hydrogen_consumption": HydrogenConsumption("hydrogen_consumption"),
    "energy_consumption": EnergyConsumption("energy_consumption"),
    "biomass_consumption": BiomassConsumption("biomass_consumption"),
    "electricity_consumption": ElectricityConsumption("electricity_consumption"),
    "equivalent_co2_emissions": EquivalentCO2Emissions("equivalent_co2_emissions"),
    "temperature": Temperature("temperature"),
    "emissions_per_rpk": EmissionsPerRPK("emissions_per_rpk"),
    "emissions_per_rtk": EmissionsPerRTK("emissions_per_rtk"),
    "dropin_fuel_consumption_liter_per_pax_100km": DropinFuelConsumptionLiterPerPax100km(
        "dropin_fuel_consumption_liter_per_pax_100km"
    ),
    "carbon_budget_consumed_share": CarbonBudgetConsumedShare("carbon_budget_consumed_share"),
    "resources_consumed_share": ResourcesConsumedShare("resources_consumed_share"),
    "biofuel_mfsp": BiofuelMfsp("biofuel_mfsp"),
    "biofuel_capex": BiofuelCapex("biofuel_capex"),
    "kerosene_market_price": KerosenePrice("kerosene_market_price"),
    "kerosene_cost": KeroseneCost("kerosene_cost"),
    "biofuel_cost": BiofuelCost("biofuel_cost"),
    "co2_cost": Co2Cost("co2_cost"),
    "co2_tax": Co2Tax("co2_tax"),
    "electricity_cost": ElectricityCost("electricity_cost"),
    "coal_cost": CoalCost("coal_cost"),
    "gas_cost": GasCost("coal_cost"),
    "liquid_hydrogen_cost": LiquidHydrogenCost("liquid_hydrogen_cost"),
    "electrolyser_capex": ElectrolyserCapex("electrolyser_capex"),
    "electrolyser_fixed_opex": ElectrolyserFixedOpex("electrolyser_fixed_opex"),
    "electrolyser_var_opex": ElectrolyserVarOpex("electrolyser_var_opex"),
    # "electrolyser_specific_electricity": ElectrolyserSpecificElectricity(
    #     "electrolyser_specific_electricity"
    # ),
    "gas_ccs_capex": GasCcsCapex("gas_ccs_capex"),
    "gas_ccs_fixed_opex": GasCcsFixedOpex("gas_ccs_fixed_opex"),
    "gas_ccs_efficiency": GasCcsEfficiency("gas_ccs_efficiency"),
    "gas_capex": GasCapex("gas_capex"),
    "gas_fixed_opex": GasFixedOpex("gas_fixed_opex"),
    "gas_efficiency": GasEfficiency("gas_efficiency"),
    "coal_ccs_capex": CoalCcsCapex("coal_ccs_capex"),
    "coal_ccs_fixed_opex": CoalCcsFixedOpex("coal_ccs_fixed_opex"),
    "coal_ccs_efficiency": CoalCcsEfficiency("coal_ccs_efficiency"),
    "coal_capex": CoalCapex("coal_capex"),
    "coal_fixed_opex": CoalFixedOpex("coal_fixed_opex"),
    "coal_efficiency": CoalEfficiency("coal_efficiency"),
    "ccs_cost": CcsCost("ccs_cost"),
    "liquefier_capex": LiquefierCapex("liquefier_capex"),
    # "liquefier_specific_electricity": LiquefierSpecificElectricity(
    #     "liquefier_specific_electricity"
    # ),
    "electrofuel_cost": ElectrofuelCost("electrofuel_cost"),
    "electrofuel_capex": ElectrofuelCapex("electrofuel_capex"),
    "electrofuel_fixed_opex": ElectrofuelFixedOpex("electrofuel_fixed_opex"),
    "electrofuel_var_opex": ElectrofuelVarOpex("electrofuel_var_opex"),
    # "electrofuel_specific_electricity": ElectrofuelSpecificElectricity("electrofuel_specific_electricity"),
    "electrofuel_specific_co2": ElectrofuelSpecificCo2("electrofuel_specific_co2"),
    "discounted_scenario_cost": DicountedScenarioCost("discounted_scenario_cost"),
    "non_discounted_scenario_cost": NonDiscountedScenarioCost("non_discounted_scenario_cost"),
    "biofuel_abatement_potential": BiofuelAbatementPotential("biofuel_abatement_potential"),
    "passenger_aircraft_doc_energy": PassengerAircraftDocEnergy("passenger_aircraft_doc_energy"),
    "passenger_aircraft_doc_non_energy_complex": PassengerAircraftDocNonEnergyComplex(
        "passenger_aircraft_doc_non_energy_complex"
    ),
    "passenger_aircraft_total_doc": PassengerAircraftTotalDoc("passenger_aircraft_total_doc"),
    "dropin_mean_mfsp": DropInMeanMfsp("dropin_mean_mfsp"),
    "passenger_aircraft_doc_carbon_tax": PassengerAircraftDocCarbonTax(
        "passenger_aircraft_doc_carbon_tax"
    ),
}



from typing import Tuple

import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class KayaFactors(AeromapsModel):
    def __init__(self, name="kaya_factors", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        ask: pd.Series = pd.Series(dtype="float64"),
        rtk: pd.Series = pd.Series(dtype="float64"),
        freight_energy_share_2019: float = 0.0,
        energy_consumption_passenger_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_passenger_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_passenger_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_biofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_electrofuel_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_kerosene_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_hydrogen_without_operations: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_consumption_freight_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_consumption: pd.Series = pd.Series(dtype="float64"),
        kerosene_emission_factor: pd.Series = pd.Series(dtype="float64"),
        biofuel_mean_emission_factor: pd.Series = pd.Series(dtype="float64"),
        electrofuel_emission_factor: pd.Series = pd.Series(dtype="float64"),
        hydrogen_mean_emission_factor: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series, pd.Series]:

        energy_per_ask_mean_without_operations = (
            energy_consumption_passenger_biofuel_without_operations
            + energy_consumption_passenger_electrofuel_without_operations
            + energy_consumption_passenger_kerosene_without_operations
            + energy_consumption_passenger_hydrogen_without_operations
        ) / ask

        energy_per_ask_mean = (
            energy_consumption_passenger_biofuel
            + energy_consumption_passenger_electrofuel
            + energy_consumption_passenger_kerosene
            + energy_consumption_passenger_hydrogen
        ) / ask

        energy_per_rtk_mean_without_operations = (
            energy_consumption_freight_biofuel_without_operations
            + energy_consumption_freight_electrofuel_without_operations
            + energy_consumption_freight_kerosene_without_operations
            + energy_consumption_freight_hydrogen_without_operations
        ) / rtk

        energy_per_rtk_mean = (
            energy_consumption_freight_biofuel
            + energy_consumption_freight_electrofuel
            + energy_consumption_freight_kerosene
            + energy_consumption_freight_hydrogen
        ) / rtk

        co2_per_energy_mean = (
            biofuel_mean_emission_factor * energy_consumption_biofuel
            + electrofuel_emission_factor * energy_consumption_electrofuel
            + kerosene_emission_factor * energy_consumption_kerosene
            + hydrogen_mean_emission_factor * energy_consumption_hydrogen
        ) / energy_consumption

        for k in range(self.historic_start_year, self.prospection_start_year):
            energy_per_ask_mean_without_operations.loc[k] = (
                energy_consumption.loc[k] / ask.loc[k] * (1 - freight_energy_share_2019 / 100)
            )
            energy_per_ask_mean.loc[k] = (
                energy_consumption.loc[k] / ask.loc[k] * (1 - freight_energy_share_2019 / 100)
            )
            energy_per_rtk_mean.loc[k] = (
                energy_consumption.loc[k] / rtk.loc[k] * freight_energy_share_2019 / 100
            )
            energy_per_rtk_mean.loc[k] = (
                energy_consumption.loc[k] / rtk.loc[k] * freight_energy_share_2019 / 100
            )
            co2_per_energy_mean.loc[k] = kerosene_emission_factor.loc[k]

        self.df.loc[
            :, "energy_per_ask_mean_without_operations"
        ] = energy_per_ask_mean_without_operations
        self.df.loc[
            :, "energy_per_rtk_mean_without_operations"
        ] = energy_per_rtk_mean_without_operations
        self.df.loc[:, "energy_per_ask_mean"] = energy_per_ask_mean
        self.df.loc[:, "energy_per_rtk_mean"] = energy_per_rtk_mean
        self.df.loc[:, "co2_per_energy_mean"] = co2_per_energy_mean

        return (
            energy_per_ask_mean_without_operations,
            energy_per_ask_mean,
            energy_per_rtk_mean_without_operations,
            energy_per_rtk_mean,
            co2_per_energy_mean,
        )


class CO2Emissions(AeromapsModel):
    def __init__(self, name="co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        rpk: pd.Series = pd.Series(dtype="float64"),
        rpk_short_range: pd.Series = pd.Series(dtype="float64"),
        rpk_medium_range: pd.Series = pd.Series(dtype="float64"),
        rpk_long_range: pd.Series = pd.Series(dtype="float64"),
        rtk: pd.Series = pd.Series(dtype="float64"),
        load_factor: pd.Series = pd.Series(dtype="float64"),
        biofuel_share: pd.Series = pd.Series(dtype="float64"),
        electrofuel_share: pd.Series = pd.Series(dtype="float64"),
        kerosene_share: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_short_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_medium_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_long_range_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_freight_dropin_fuel: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_short_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_medium_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_long_range_hydrogen: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
        ask_short_range_dropin_fuel_share: pd.Series = pd.Series(dtype="float64"),
        ask_medium_range_dropin_fuel_share: pd.Series = pd.Series(dtype="float64"),
        ask_long_range_dropin_fuel_share: pd.Series = pd.Series(dtype="float64"),
        rtk_dropin_fuel_share: pd.Series = pd.Series(dtype="float64"),
        ask_short_range_hydrogen_share: pd.Series = pd.Series(dtype="float64"),
        ask_medium_range_hydrogen_share: pd.Series = pd.Series(dtype="float64"),
        ask_long_range_hydrogen_share: pd.Series = pd.Series(dtype="float64"),
        rtk_hydrogen_share: pd.Series = pd.Series(dtype="float64"),
        kerosene_emission_factor: pd.Series = pd.Series(dtype="float64"),
        biofuel_mean_emission_factor: pd.Series = pd.Series(dtype="float64"),
        electrofuel_emission_factor: pd.Series = pd.Series(dtype="float64"),
        hydrogen_mean_emission_factor: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_mean: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_mean: pd.Series = pd.Series(dtype="float64"),
        co2_per_energy_mean: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series, pd.Series, pd.Series]:
        """CO2 emissions calculation."""
        # To be improved

        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "co2_emissions_short_range"] = (
                rpk_short_range.loc[k]
                / (load_factor.loc[k] / 100)
                * energy_per_ask_mean.loc[k]
                * co2_per_energy_mean.loc[k]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_medium_range"] = (
                rpk_medium_range.loc[k]
                / (load_factor.loc[k] / 100)
                * energy_per_ask_mean.loc[k]
                * co2_per_energy_mean.loc[k]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_long_range"] = (
                rpk_long_range.loc[k]
                / (load_factor.loc[k] / 100)
                * energy_per_ask_mean.loc[k]
                * co2_per_energy_mean.loc[k]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_freight"] = (
                rtk.loc[k] * energy_per_rtk_mean.loc[k] * co2_per_energy_mean.loc[k] * 10 ** (-12)
            )

        # Short range
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "co2_emissions_short_range"] = (
                rpk_short_range.loc[k]
                / (load_factor.loc[k] / 100)
                * (
                    ask_short_range_dropin_fuel_share.loc[k]
                    / 100
                    * (
                        biofuel_share.loc[k]
                        / 100
                        * energy_per_ask_short_range_dropin_fuel.loc[k]
                        * biofuel_mean_emission_factor.loc[k]
                        + electrofuel_share.loc[k]
                        / 100
                        * energy_per_ask_short_range_dropin_fuel.loc[k]
                        * electrofuel_emission_factor.loc[k]
                        + kerosene_share.loc[k]
                        / 100
                        * energy_per_ask_short_range_dropin_fuel.loc[k]
                        * kerosene_emission_factor.loc[k]
                    )
                    + ask_short_range_hydrogen_share.loc[k]
                    / 100
                    * energy_per_ask_short_range_hydrogen.loc[k]
                    * hydrogen_mean_emission_factor.loc[k]
                )
                * 10 ** (-12)
            )

        # Medium range
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "co2_emissions_medium_range"] = (
                rpk_medium_range.loc[k]
                / (load_factor.loc[k] / 100)
                * (
                    ask_medium_range_dropin_fuel_share.loc[k]
                    / 100
                    * (
                        biofuel_share.loc[k]
                        / 100
                        * energy_per_ask_medium_range_dropin_fuel.loc[k]
                        * biofuel_mean_emission_factor.loc[k]
                        + electrofuel_share.loc[k]
                        / 100
                        * energy_per_ask_medium_range_dropin_fuel.loc[k]
                        * electrofuel_emission_factor.loc[k]
                        + kerosene_share.loc[k]
                        / 100
                        * energy_per_ask_medium_range_dropin_fuel.loc[k]
                        * kerosene_emission_factor.loc[k]
                    )
                    + ask_medium_range_hydrogen_share.loc[k]
                    / 100
                    * energy_per_ask_medium_range_hydrogen.loc[k]
                    * hydrogen_mean_emission_factor.loc[k]
                )
                * 10 ** (-12)
            )

        # Long range
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "co2_emissions_long_range"] = (
                rpk_long_range.loc[k]
                / (load_factor.loc[k] / 100)
                * (
                    ask_long_range_dropin_fuel_share.loc[k]
                    / 100
                    * (
                        biofuel_share.loc[k]
                        / 100
                        * energy_per_ask_long_range_dropin_fuel.loc[k]
                        * biofuel_mean_emission_factor.loc[k]
                        + electrofuel_share.loc[k]
                        / 100
                        * energy_per_ask_long_range_dropin_fuel.loc[k]
                        * electrofuel_emission_factor.loc[k]
                        + kerosene_share.loc[k]
                        / 100
                        * energy_per_ask_long_range_dropin_fuel.loc[k]
                        * kerosene_emission_factor.loc[k]
                    )
                    + ask_long_range_hydrogen_share.loc[k]
                    / 100
                    * energy_per_ask_long_range_hydrogen.loc[k]
                    * hydrogen_mean_emission_factor.loc[k]
                )
                * 10 ** (-12)
            )

        # Freight
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "co2_emissions_freight"] = (
                rtk.loc[k]
                * (
                    rtk_dropin_fuel_share.loc[k]
                    / 100
                    * (
                        biofuel_share.loc[k]
                        / 100
                        * energy_per_rtk_freight_dropin_fuel.loc[k]
                        * biofuel_mean_emission_factor.loc[k]
                        + electrofuel_share.loc[k]
                        / 100
                        * energy_per_rtk_freight_dropin_fuel.loc[k]
                        * electrofuel_emission_factor.loc[k]
                        + kerosene_share.loc[k]
                        / 100
                        * energy_per_rtk_freight_dropin_fuel.loc[k]
                        * kerosene_emission_factor.loc[k]
                    )
                    + rtk_hydrogen_share.loc[k]
                    / 100
                    * energy_per_rtk_freight_hydrogen.loc[k]
                    * hydrogen_mean_emission_factor.loc[k]
                )
                * 10 ** (-12)
            )

        # Passenger and total
        for k in range(self.historic_start_year, self.end_year + 1):
            self.df.loc[k, "co2_emissions_passenger"] = (
                self.df.loc[k, "co2_emissions_short_range"]
                + self.df.loc[k, "co2_emissions_medium_range"]
                + self.df.loc[k, "co2_emissions_long_range"]
            )
            self.df.loc[k, "co2_emissions"] = (
                self.df.loc[k, "co2_emissions_passenger"] + self.df.loc[k, "co2_emissions_freight"]
            )

        co2_emissions_short_range = self.df["co2_emissions_short_range"]
        co2_emissions_medium_range = self.df["co2_emissions_medium_range"]
        co2_emissions_long_range = self.df["co2_emissions_long_range"]
        co2_emissions_freight = self.df["co2_emissions_freight"]
        co2_emissions_passenger = self.df["co2_emissions_passenger"]
        co2_emissions = self.df["co2_emissions"]

        return (
            co2_emissions_short_range,
            co2_emissions_medium_range,
            co2_emissions_long_range,
            co2_emissions_passenger,
            co2_emissions_freight,
            co2_emissions,
        )


class CumulativeCO2Emissions(AeromapsModel):
    def __init__(self, name="cumulative_co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        co2_emissions: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series]:

        self.df.loc[self.prospection_start_year - 1, "cumulative_co2_emissions"] = 0.0
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "cumulative_co2_emissions"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions"] + co2_emissions.loc[k] / 1000
            )

        cumulative_co2_emissions = self.df["cumulative_co2_emissions"]

        return cumulative_co2_emissions


class DetailedCo2Emissions(AeromapsModel):
    def __init__(self, name="detailed_co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        rpk_reference: pd.Series = pd.Series(dtype="float64"),
        rtk_reference: pd.Series = pd.Series(dtype="float64"),
        rpk: pd.Series = pd.Series(dtype="float64"),
        rtk: pd.Series = pd.Series(dtype="float64"),
        load_factor: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_mean: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_mean: pd.Series = pd.Series(dtype="float64"),
        energy_per_ask_mean_without_operations: pd.Series = pd.Series(dtype="float64"),
        energy_per_rtk_mean_without_operations: pd.Series = pd.Series(dtype="float64"),
        co2_per_energy_mean: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series, pd.Series, pd.Series]:
        for k in range(self.prospection_start_year - 1, self.end_year + 1):
            self.df.loc[k, "co2_emissions_2019technology_baseline3"] = (
                rpk_reference.loc[k]
                * energy_per_ask_mean_without_operations.loc[self.prospection_start_year - 1]
                * energy_per_ask_mean.loc[self.prospection_start_year - 1]
                / energy_per_ask_mean_without_operations.loc[self.prospection_start_year - 1]
                / (load_factor.loc[self.prospection_start_year - 1] / 100)
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            ) + (
                rtk_reference.loc[k]
                * energy_per_rtk_mean_without_operations.loc[self.prospection_start_year - 1]
                * energy_per_rtk_mean.loc[self.prospection_start_year - 1]
                / energy_per_rtk_mean_without_operations.loc[self.prospection_start_year - 1]
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_2019technology"] = (
                rpk.loc[k]
                * energy_per_ask_mean_without_operations.loc[self.prospection_start_year - 1]
                * energy_per_ask_mean.loc[self.prospection_start_year - 1]
                / energy_per_ask_mean_without_operations.loc[self.prospection_start_year - 1]
                / (load_factor.loc[self.prospection_start_year - 1] / 100)
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            ) + (
                rtk.loc[k]
                * energy_per_rtk_mean_without_operations.loc[self.prospection_start_year - 1]
                * energy_per_rtk_mean.loc[self.prospection_start_year - 1]
                / energy_per_rtk_mean_without_operations.loc[self.prospection_start_year - 1]
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_including_aircraft_efficiency"] = (
                rpk.loc[k]
                * energy_per_ask_mean_without_operations.loc[k]
                * energy_per_ask_mean.loc[self.prospection_start_year - 1]
                / energy_per_ask_mean_without_operations.loc[self.prospection_start_year - 1]
                / (load_factor.loc[self.prospection_start_year - 1] / 100)
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            ) + (
                rtk.loc[k]
                * energy_per_rtk_mean_without_operations.loc[k]
                * energy_per_rtk_mean.loc[self.prospection_start_year - 1]
                / energy_per_rtk_mean_without_operations.loc[self.prospection_start_year - 1]
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_including_operations"] = (
                rpk.loc[k]
                * energy_per_ask_mean_without_operations.loc[k]
                * energy_per_ask_mean.loc[k]
                / energy_per_ask_mean_without_operations.loc[k]
                / (load_factor.loc[self.prospection_start_year - 1] / 100)
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            ) + (
                rtk.loc[k]
                * energy_per_rtk_mean_without_operations.loc[k]
                * energy_per_rtk_mean.loc[k]
                / energy_per_rtk_mean_without_operations.loc[k]
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_including_load_factor"] = (
                rpk.loc[k]
                * energy_per_ask_mean_without_operations.loc[k]
                * energy_per_ask_mean.loc[k]
                / energy_per_ask_mean_without_operations.loc[k]
                / (load_factor.loc[k] / 100)
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            ) + (
                rtk.loc[k]
                * energy_per_rtk_mean_without_operations.loc[k]
                * energy_per_rtk_mean.loc[k]
                / energy_per_rtk_mean_without_operations.loc[k]
                * co2_per_energy_mean.loc[self.prospection_start_year - 1]
                * 10 ** (-12)
            )
            self.df.loc[k, "co2_emissions_including_energy"] = (
                rpk.loc[k]
                * energy_per_ask_mean_without_operations.loc[k]
                * energy_per_ask_mean.loc[k]
                / energy_per_ask_mean_without_operations.loc[k]
                / (load_factor.loc[k] / 100)
                * co2_per_energy_mean.loc[k]
                * 10 ** (-12)
            ) + (
                rtk.loc[k]
                * energy_per_rtk_mean_without_operations.loc[k]
                * energy_per_rtk_mean.loc[k]
                / energy_per_rtk_mean_without_operations.loc[k]
                * co2_per_energy_mean.loc[k]
                * 10 ** (-12)
            )

        co2_emissions_2019technology_baseline3 = self.df["co2_emissions_2019technology_baseline3"]
        co2_emissions_2019technology = self.df["co2_emissions_2019technology"]
        co2_emissions_including_aircraft_efficiency = self.df[
            "co2_emissions_including_aircraft_efficiency"
        ]
        co2_emissions_including_operations = self.df["co2_emissions_including_operations"]
        co2_emissions_including_load_factor = self.df["co2_emissions_including_load_factor"]
        co2_emissions_including_energy = self.df["co2_emissions_including_energy"]

        return (
            co2_emissions_2019technology_baseline3,
            co2_emissions_2019technology,
            co2_emissions_including_aircraft_efficiency,
            co2_emissions_including_operations,
            co2_emissions_including_load_factor,
            co2_emissions_including_energy,
        )


class DetailedCumulativeCO2Emissions(AeromapsModel):
    def __init__(self, name="detailed_cumulative_co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        co2_emissions_2019technology_baseline3: pd.Series = pd.Series(dtype="float64"),
        co2_emissions_2019technology: pd.Series = pd.Series(dtype="float64"),
        co2_emissions_including_load_factor: pd.Series = pd.Series(dtype="float64"),
        co2_emissions_including_energy: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series]:

        self.df.loc[
            self.prospection_start_year - 1, "cumulative_co2_emissions_2019technology_baseline3"
        ] = 0.0
        self.df.loc[
            self.prospection_start_year - 1, "cumulative_co2_emissions_2019technology"
        ] = 0.0
        self.df.loc[
            self.prospection_start_year - 1, "cumulative_co2_emissions_including_load_factor"
        ] = 0.0
        self.df.loc[
            self.prospection_start_year - 1, "cumulative_co2_emissions_including_energy"
        ] = 0.0

        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "cumulative_co2_emissions_2019technology_baseline3"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions_2019technology_baseline3"]
                + co2_emissions_2019technology_baseline3.loc[k] / 1000
            )
            self.df.loc[k, "cumulative_co2_emissions_2019technology"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions_2019technology"]
                + co2_emissions_2019technology.loc[k] / 1000
            )
            self.df.loc[k, "cumulative_co2_emissions_including_load_factor"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions_including_load_factor"]
                + co2_emissions_including_load_factor.loc[k] / 1000
            )
            self.df.loc[k, "cumulative_co2_emissions_including_energy"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions_including_energy"]
                + co2_emissions_including_energy.loc[k] / 1000
            )

        cumulative_co2_emissions_2019technology_baseline3 = self.df[
            "cumulative_co2_emissions_2019technology_baseline3"
        ]
        cumulative_co2_emissions_2019technology = self.df["cumulative_co2_emissions_2019technology"]
        cumulative_co2_emissions_including_load_factor = self.df[
            "cumulative_co2_emissions_including_load_factor"
        ]
        cumulative_co2_emissions_including_energy = self.df[
            "cumulative_co2_emissions_including_energy"
        ]

        return (
            cumulative_co2_emissions_2019technology_baseline3,
            cumulative_co2_emissions_2019technology,
            cumulative_co2_emissions_including_load_factor,
            cumulative_co2_emissions_including_energy,
        )



from typing import Tuple

import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class OperationsSimple(AeromapsModel):
    def __init__(self, name="operations_simple", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        operations_final_gain: float,
        operations_start_year: float,
        operations_duration: float,
    ) -> pd.Series:
        """Operations gain for efficiency calculation."""

        transition_year = operations_start_year + operations_duration / 2
        operations_limit = 0.02 * operations_final_gain
        operations_parameter = np.log(100 / 2 - 1) / (operations_duration / 2)
        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "operations_gain"] = 0
        for k in range(self.prospection_start_year - 1, self.end_year + 1):
            if (
                operations_final_gain / (1 + np.exp(-operations_parameter * (k - transition_year)))
                < operations_limit
            ):
                self.df.loc[k, "operations_gain"] = 0
            else:
                self.df.loc[k, "operations_gain"] = operations_final_gain / (
                    1 + np.exp(-operations_parameter * (k - transition_year))
                )

        operations_gain = self.df["operations_gain"]

        return operations_gain



from typing import Tuple

import numpy as np
import pandas as pd
from scipy.interpolate import interp1d

from aeromaps.models.base import AeromapsModel


class EnergyIntensity(AeromapsModel):
    def __init__(self, name="passenger_energy_intensity", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        energy_per_ask_without_operations_short_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_medium_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_long_range_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_rtk_without_operations_freight_dropin_fuel: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_short_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_medium_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_ask_without_operations_long_range_hydrogen: pd.Series = pd.Series(
            dtype="float64"
        ),
        energy_per_rtk_without_operations_freight_hydrogen: pd.Series = pd.Series(dtype="float64"),
        operations_gain: pd.Series = pd.Series(dtype="float64"),
        operations_contrails_overconsumption: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[
        pd.Series, pd.Series, pd.Series, pd.Series, pd.Series, pd.Series, pd.Series, pd.Series
    ]:
        """Energy consumption per ASK (with operations) calculation using simple models."""

        energy_per_ask_short_range_dropin_fuel = (
            energy_per_ask_without_operations_short_range_dropin_fuel
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_ask_medium_range_dropin_fuel = (
            energy_per_ask_without_operations_medium_range_dropin_fuel
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_ask_long_range_dropin_fuel = (
            energy_per_ask_without_operations_long_range_dropin_fuel
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_rtk_freight_dropin_fuel = (
            energy_per_rtk_without_operations_freight_dropin_fuel
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_ask_short_range_hydrogen = (
            energy_per_ask_without_operations_short_range_hydrogen
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_ask_medium_range_hydrogen = (
            energy_per_ask_without_operations_medium_range_hydrogen
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_ask_long_range_hydrogen = (
            energy_per_ask_without_operations_long_range_hydrogen
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )
        energy_per_rtk_freight_hydrogen = (
            energy_per_rtk_without_operations_freight_hydrogen
            * (1 - operations_gain / 100)
            * (1 + operations_contrails_overconsumption / 100)
        )

        self.df.loc[
            :, "energy_per_ask_short_range_dropin_fuel"
        ] = energy_per_ask_short_range_dropin_fuel
        self.df.loc[
            :, "energy_per_ask_medium_range_dropin_fuel"
        ] = energy_per_ask_medium_range_dropin_fuel
        self.df.loc[
            :, "energy_per_ask_long_range_dropin_fuel"
        ] = energy_per_ask_long_range_dropin_fuel
        self.df.loc[:, "energy_per_rtk_freight_dropin_fuel"] = energy_per_rtk_freight_dropin_fuel
        self.df.loc[:, "energy_per_ask_short_range_hydrogen"] = energy_per_ask_short_range_hydrogen
        self.df.loc[
            :, "energy_per_ask_medium_range_hydrogen"
        ] = energy_per_ask_medium_range_hydrogen
        self.df.loc[:, "energy_per_ask_long_range_hydrogen"] = energy_per_ask_long_range_hydrogen
        self.df.loc[:, "energy_per_rtk_freight_hydrogen"] = energy_per_rtk_freight_hydrogen

        return (
            energy_per_ask_short_range_dropin_fuel,
            energy_per_ask_medium_range_dropin_fuel,
            energy_per_ask_long_range_dropin_fuel,
            energy_per_rtk_freight_dropin_fuel,
            energy_per_ask_short_range_hydrogen,
            energy_per_ask_medium_range_hydrogen,
            energy_per_ask_long_range_hydrogen,
            energy_per_rtk_freight_hydrogen,
        )



import pandas as pd
import numpy as np
from scipy.interpolate import interp1d

from aeromaps.models.constants import ModelType


class AeromapsModel(object):
    def __init__(
        self,
        name,
        parameters=None,
    ):
        self.name = name
        self.parameters = parameters
        self.float_outputs = {}
        if self.parameters is not None:
            self._initialize_df()

    def _initialize_df(self):
        self.historic_start_year = self.parameters.historic_start_year
        self.prospection_start_year = self.parameters.prospection_start_year
        self.end_year = self.parameters.end_year
        self.df: pd.DataFrame = pd.DataFrame(
            index=range(self.historic_start_year, self.end_year + 1)
        )
        self.years = np.linspace(self.historic_start_year, self.end_year, len(self.df.index))


class LogisticFunctionYearSeries(AeromapsModel):
    def __init__(self, maximum_value=None, growth_rate=None, midpoint_year=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.maximum_value = maximum_value
        self.growth_rate = growth_rate
        self.midpoint_year = midpoint_year

    def _compute(self, maximum_value=None, growth_rate=None, midpoint_year=None):
        if not maximum_value:
            maximum_value = self.maximum_value
        if not growth_rate:
            growth_rate = self.growth_rate
        if not midpoint_year:
            midpoint_year = self.midpoint_year
        x = np.linspace(self.prospection_start_year, self.end_year, len(self.df.index))
        y = maximum_value / (1 + np.exp(-growth_rate * (x - midpoint_year)))
        self.df[self.name] = y
        return y


class ExponentialGrowthYearSeries(AeromapsModel):
    def __init__(
        self, initial_value=0.0, growth_rates={2030: 3.1, 2040: 3.0, 2050: 2.9}, *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.initial_value = initial_value
        self.growth_rates = growth_rates

    def _compute(self, initial_value=None, growth_rates=None):
        if not initial_value:
            initial_value = self.initial_value
        if not growth_rates:
            growth_rates = self.growth_rates
        x = self.df.index
        y = np.ones(len(x))
        self.df[self.name] = y
        self.df.loc[self.start_year, self.name] = initial_value
        for year in x:
            if year != self.start_year:
                self.df.loc[year, self.name] = self.df.loc[year - 1, self.name] * (
                    1 + self._get_growth_rate(year, growth_rates=growth_rates) / 100
                )

        return y

    def _get_growth_rate(self, year, growth_rates=None):
        if not growth_rates:
            growth_rates = self.growth_rates
        for key, growth_rate in growth_rates.items():
            if year <= key:
                actual_growth_rate = growth_rate

        return actual_growth_rate


class InterpAeromapsModel(AeromapsModel):
    def __init__(
        self,
        year_values={2020: 100.0, 2030: 80.0, 2040: 50.0, 2050: 20.0},
        model_type=ModelType.LINEAR,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.year_values = year_values
        self.model_type = model_type

    def _compute(self):
        reference_years, reference_values = self.year_values.items()
        interpolation = interp1d(reference_years, reference_values, kind=self.model_type)

        values = interpolation(self.years)

        return values



from typing import Tuple

import numpy as np
import pandas as pd
from scipy.interpolate import interp1d

from aeromaps.models.base import AeromapsModel


class ShortRangeDistribution(AeromapsModel):
    def __init__(self, name="short_range_distribution", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        short_range_basicturbofan_share_2019: float = 0.0,
        short_range_basicturbofan_share_2030: float = 0.0,
        short_range_basicturbofan_share_2040: float = 0.0,
        short_range_basicturbofan_share_2050: float = 0.0,
        short_range_regionalturboprop_share_2019: float = 0.0,
        short_range_regionalturboprop_share_2030: float = 0.0,
        short_range_regionalturboprop_share_2040: float = 0.0,
        short_range_regionalturboprop_share_2050: float = 0.0,
    ) -> Tuple[pd.Series, pd.Series, pd.Series,]:
        """Short range distribution calculation."""

        reference_years = [2019, 2030, 2040, self.end_year]

        # Basic Turbofan
        reference_values_basicturbofan = [
            short_range_basicturbofan_share_2019,
            short_range_basicturbofan_share_2030,
            short_range_basicturbofan_share_2040,
            short_range_basicturbofan_share_2050,
        ]
        short_range_basicturbofan_share_function = interp1d(
            reference_years, reference_values_basicturbofan, kind="linear"
        )
        for k in range(self.prospection_start_year - 1, self.end_year + 1):
            self.df.loc[
                k, "short_range_basicturbofan_share"
            ] = short_range_basicturbofan_share_function(k)

        # Regional Turboprop
        reference_values_regionalturboprop = [
            short_range_regionalturboprop_share_2019,
            short_range_regionalturboprop_share_2030,
            short_range_regionalturboprop_share_2040,
            short_range_regionalturboprop_share_2050,
        ]
        short_range_regionalturboprop_share_function = interp1d(
            reference_years, reference_values_regionalturboprop, kind="linear"
        )
        for k in range(self.prospection_start_year - 1, self.end_year + 1):
            self.df.loc[
                k, "short_range_regionalturboprop_share"
            ] = short_range_regionalturboprop_share_function(k)

        short_range_basicturbofan_share = self.df["short_range_basicturbofan_share"]
        short_range_regionalturboprop_share = self.df["short_range_regionalturboprop_share"]

        # Regional Turbofan
        short_range_regionalturbofan_share = (
            100 - short_range_basicturbofan_share - short_range_regionalturboprop_share
        )
        self.df.loc[:, "short_range_regionalturbofan_share"] = short_range_regionalturbofan_share

        return (
            short_range_basicturbofan_share,
            short_range_regionalturboprop_share,
            short_range_regionalturbofan_share,
        )


class RPKShortRange(AeromapsModel):
    def __init__(self, name="rpk_short_range", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        rpk_short_range: pd.Series = pd.Series(dtype="float64"),
        short_range_basicturbofan_share: pd.Series = pd.Series(dtype="float64"),
        short_range_regionalturboprop_share: pd.Series = pd.Series(dtype="float64"),
        short_range_regionalturbofan_share: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, float, float, float, float, float, float,]:
        """RPK short range calculation."""

        rpk_short_range_basicturbofan = rpk_short_range * short_range_basicturbofan_share / 100
        rpk_short_range_regionalturboprop = (
            rpk_short_range * short_range_regionalturboprop_share / 100
        )
        rpk_short_range_regionalturbofan = (
            rpk_short_range * short_range_regionalturbofan_share / 100
        )

        self.df.loc[:, "rpk_short_range_basicturbofan"] = rpk_short_range_basicturbofan
        self.df.loc[:, "rpk_short_range_regionalturboprop"] = rpk_short_range_regionalturboprop
        self.df.loc[:, "rpk_short_range_regionalturbofan"] = rpk_short_range_regionalturbofan

        # Compound Annual Growth Rate (CAGR)
        cagr_rpk_short_range_basicturbofan = 100 * (
            (
                self.df.loc[self.end_year, "rpk_short_range_basicturbofan"]
                / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_basicturbofan"]
            )
            ** (1 / (self.end_year - self.prospection_start_year))
            - 1
        )
        cagr_rpk_short_range_regionalturboprop = 100 * (
            (
                self.df.loc[self.end_year, "rpk_short_range_regionalturboprop"]
                / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_regionalturboprop"]
            )
            ** (1 / (self.end_year - self.prospection_start_year))
            - 1
        )
        cagr_rpk_short_range_regionalturbofan = 100 * (
            (
                self.df.loc[self.end_year, "rpk_short_range_regionalturbofan"]
                / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_regionalturbofan"]
            )
            ** (1 / (self.end_year - self.prospection_start_year))
            - 1
        )

        # Prospective evolution of RPK (between prospection_start_year-1 and end_year)
        prospective_evolution_rpk_short_range_basicturbofan = 100 * (
            self.df.loc[self.end_year, "rpk_short_range_basicturbofan"]
            / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_basicturbofan"]
            - 1
        )
        prospective_evolution_rpk_short_range_regionalturboprop = 100 * (
            self.df.loc[self.end_year, "rpk_short_range_regionalturboprop"]
            / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_regionalturboprop"]
            - 1
        )
        prospective_evolution_rpk_short_range_regionalturbofan = 100 * (
            self.df.loc[self.end_year, "rpk_short_range_regionalturbofan"]
            / self.df.loc[self.prospection_start_year - 1, "rpk_short_range_regionalturbofan"]
            - 1
        )

        self.float_outputs[
            "cagr_rpk_short_range_basicturbofan"
        ] = cagr_rpk_short_range_basicturbofan
        self.float_outputs[
            "cagr_rpk_short_range_regionalturboprop"
        ] = cagr_rpk_short_range_regionalturboprop
        self.float_outputs[
            "cagr_rpk_short_range_regionalturbofan"
        ] = cagr_rpk_short_range_regionalturbofan
        self.float_outputs[
            "prospective_evolution_rpk_short_range_basicturbofan"
        ] = prospective_evolution_rpk_short_range_basicturbofan
        self.float_outputs[
            "prospective_evolution_rpk_short_range_regionalturboprop"
        ] = prospective_evolution_rpk_short_range_regionalturboprop
        self.float_outputs[
            "prospective_evolution_rpk_short_range_regionalturbofan"
        ] = prospective_evolution_rpk_short_range_regionalturbofan

        return (
            rpk_short_range_basicturbofan,
            rpk_short_range_regionalturboprop,
            rpk_short_range_regionalturbofan,
            cagr_rpk_short_range_basicturbofan,
            cagr_rpk_short_range_regionalturboprop,
            cagr_rpk_short_range_regionalturbofan,
            prospective_evolution_rpk_short_range_basicturbofan,
            prospective_evolution_rpk_short_range_regionalturboprop,
            prospective_evolution_rpk_short_range_regionalturbofan,
        )
